@@ -26,11 +26,11 @@ import random
 import threading
 import warnings
 from collections import OrderedDict
-from typing import Callable, Dict, List, Optional, Set, TYPE_CHECKING, Tuple, Union, cast
+from typing import Callable, Dict, Iterable, List, Optional, Set, TYPE_CHECKING, Tuple, Union, cast
 
 from .._dns import DNSAddress, DNSPointer, DNSQuestion, DNSQuestionType, DNSRecord
 from .._logger import log
-from .._protocol import DNSOutgoing
+from .._protocol.outgoing import DNSOutgoing
 from .._services import (
     ServiceListener,
     ServiceStateChange,
@@ -38,8 +38,7 @@ from .._services import (
     SignalRegistrationInterface,
 )
 from .._updates import RecordUpdate, RecordUpdateListener
-from .._utils.asyncio import get_best_available_queue
-from .._utils.name import service_type_name
+from .._utils.name import possible_types, service_type_name
 from .._utils.time import current_time_millis, millis_to_seconds
 from ..const import (
     _BROWSER_BACKOFF_LIMIT,
@@ -145,11 +144,11 @@ def generate_service_query(
     for type_ in types_:
         question = DNSQuestion(type_, _TYPE_PTR, _CLASS_IN)
         question.unicast = qu_question
-        known_answers = set(
+        known_answers = {
             cast(DNSPointer, record)
             for record in zc.cache.get_all_by_details(type_, _TYPE_PTR, _CLASS_IN)
             if not record.is_stale(now)
-        )
+        }
         if not qu_question and zc.question_history.suppresses(
             question, now, cast(Set[DNSRecord], known_answers)
         ):
@@ -293,7 +292,7 @@ class _ServiceBrowserBase(RecordUpdateListener):
         self._pending_handlers: OrderedDict[Tuple[str, str], ServiceStateChange] = OrderedDict()
         self._service_state_changed = Signal()
         self.query_scheduler = QueryScheduler(self.types, delay, _FIRST_QUERY_DELAY_RANDOM_INTERVAL)
-        self.queue: Optional[queue.Queue] = None
+        self.queue: Optional[queue.SimpleQueue] = None
         self.done = False
         self._first_request: bool = True
         self._next_send_timer: Optional[asyncio.TimerHandle] = None
@@ -325,9 +324,9 @@ class _ServiceBrowserBase(RecordUpdateListener):
     def service_state_changed(self) -> SignalRegistrationInterface:
         return self._service_state_changed.registration_interface
 
-    def _record_matching_type(self, record: DNSRecord) -> Optional[str]:
-        """Return the type if the record matches one of the types we are browsing."""
-        return next((type_ for type_ in self.types if record.name.endswith(type_)), None)
+    def _names_matching_types(self, names: Iterable[str]) -> List[Tuple[str, str]]:
+        """Return the type and name for records matching the types we are browsing."""
+        return [(type_, name) for name in names for type_ in self.types.intersection(possible_types(name))]
 
     def _enqueue_callback(
         self,
@@ -353,14 +352,13 @@ class _ServiceBrowserBase(RecordUpdateListener):
     ) -> None:
         """Process a single record update from a batch of updates."""
         if isinstance(record, DNSPointer):
-            if record.name not in self.types:
-                return
-            if old_record is None:
-                self._enqueue_callback(ServiceStateChange.Added, record.name, record.alias)
-            elif record.is_expired(now):
-                self._enqueue_callback(ServiceStateChange.Removed, record.name, record.alias)
-            else:
-                self.reschedule_type(record.name, record.get_expiration_time(_EXPIRE_REFRESH_TIME_PERCENT))
+            for type_ in self.types.intersection(possible_types(record.name)):
+                if old_record is None:
+                    self._enqueue_callback(ServiceStateChange.Added, type_, record.alias)
+                elif record.is_expired(now):
+                    self._enqueue_callback(ServiceStateChange.Removed, type_, record.alias)
+                else:
+                    self.reschedule_type(type_, now, record.get_expiration_time(_EXPIRE_REFRESH_TIME_PERCENT))
             return
 
         # If its expired or already exists in the cache it cannot be updated.
@@ -369,17 +367,14 @@ class _ServiceBrowserBase(RecordUpdateListener):
 
         if isinstance(record, DNSAddress):
             # Iterate through the DNSCache and callback any services that use this address
-            for service in self.zc.cache.async_entries_with_server(record.name):
-                type_ = self._record_matching_type(service)
-                if type_:
-                    self._enqueue_callback(ServiceStateChange.Updated, type_, service.name)
-                    break
-
+            for type_, name in self._names_matching_types(
+                {service.name for service in self.zc.cache.async_entries_with_server(record.name)}
+            ):
+                self._enqueue_callback(ServiceStateChange.Updated, type_, name)
             return
 
-        type_ = self._record_matching_type(record)
-        if type_:
-            self._enqueue_callback(ServiceStateChange.Updated, type_, record.name)
+        for type_, name in self._names_matching_types((record.name,)):
+            self._enqueue_callback(ServiceStateChange.Updated, type_, name)
 
     def async_update_records(self, zc: 'Zeroconf', now: float, records: List[RecordUpdate]) -> None:
         """Callback invoked by Zeroconf when new information arrives.
@@ -431,9 +426,8 @@ class _ServiceBrowserBase(RecordUpdateListener):
         self._cancel_send_timer()
         self.zc.async_remove_listener(self)
 
-    def _generate_ready_queries(self, first_request: bool) -> List[DNSOutgoing]:
+    def _generate_ready_queries(self, first_request: bool, now: float) -> List[DNSOutgoing]:
         """Generate the service browser query for any type that is due."""
-        now = current_time_millis()
         ready_types = self.query_scheduler.process_ready_types(now)
         if not ready_types:
             return []
@@ -447,42 +441,42 @@ class _ServiceBrowserBase(RecordUpdateListener):
 
     async def _async_start_query_sender(self) -> None:
         """Start scheduling queries."""
-        await self.zc.async_wait_for_start()
-        self._async_send_ready_queries()
-        self._async_schedule_next()
+        if not self.zc.started:
+            await self.zc.async_wait_for_start()
+        self._async_send_ready_queries_schedule_next()
 
     def _cancel_send_timer(self) -> None:
         """Cancel the next send."""
         if self._next_send_timer:
             self._next_send_timer.cancel()
 
-    def reschedule_type(self, type_: str, next_time: float) -> None:
+    def reschedule_type(self, type_: str, now: float, next_time: float) -> None:
         """Reschedule a type to be refreshed in the future."""
         if self.query_scheduler.reschedule_type(type_, next_time):
             self._cancel_send_timer()
-            self._async_schedule_next()
-        self._async_send_ready_queries()
+            self._async_schedule_next(now)
+        self._async_send_ready_queries(now)
 
-    def _async_send_ready_queries(self) -> None:
+    def _async_send_ready_queries(self, now: float) -> None:
         """Send any ready queries."""
-        if self.done or self.zc.done:
-            return
-
-        outs = self._generate_ready_queries(self._first_request)
+        outs = self._generate_ready_queries(self._first_request, now)
         if outs:
             self._first_request = False
             for out in outs:
                 self.zc.async_send(out, addr=self.addr, port=self.port)
 
     def _async_send_ready_queries_schedule_next(self) -> None:
-        """Send ready queries and schedule next one."""
-        self._async_send_ready_queries()
-        self._async_schedule_next()
+        """Send ready queries and schedule next one checking for done first."""
+        if self.done or self.zc.done:
+            return
+        now = current_time_millis()
+        self._async_send_ready_queries(now)
+        self._async_schedule_next(now)
 
-    def _async_schedule_next(self) -> None:
+    def _async_schedule_next(self, now: float) -> None:
         """Scheule the next time."""
         assert self.zc.loop is not None
-        delay = millis_to_seconds(self.query_scheduler.millis_to_wait(current_time_millis()))
+        delay = millis_to_seconds(self.query_scheduler.millis_to_wait(now))
         self._next_send_timer = self.zc.loop.call_later(delay, self._async_send_ready_queries_schedule_next)
 
 
@@ -512,11 +506,11 @@ class ServiceBrowser(_ServiceBrowserBase, threading.Thread):
         # Add the queue before the listener is installed in _setup
         # to ensure that events run in the dedicated thread and do
         # not block the event loop
-        self.queue = get_best_available_queue()
+        self.queue = queue.SimpleQueue()
         self.daemon = True
         self.start()
         zc.loop.call_soon_threadsafe(self._async_start)
-        self.name = "zeroconf-ServiceBrowser-%s-%s" % (
+        self.name = "zeroconf-ServiceBrowser-{}-{}".format(
             '-'.join([type_[:-7] for type_ in self.types]),
             getattr(self, 'native_id', self.ident),
         )
